#ifndef C99TREE_CC_H
#define C99TREE_CC_H
#include <stdio.h>

typedef struct c99c_gtx_list_s c99c_gtx_list_t;
typedef struct c99tree_cc_s c99tree_cc_t;
typedef struct loc_s loc_t;
typedef struct c99c_scope_s c99c_scope_t;

typedef enum print_state {
	PS_HEAD,      /* file header (before first opening) */
	PS_OPENING,   /* tree node opening (all children will follow recursively) */
	PS_CLOSING,   /* tree node closed (all children already printed) */
	PS_TAIL       /* file footer (after last closing) */
} print_state_t;

#include <genht/htsp.h>
#include <genht/htss.h>
#include <porty/c99tree-cc/tree.h>

#include <gentrex/src/helper_map_az.h>
#include <gentrex/src/helper_intop.h>

typedef enum verbose_options_e {
	VO_TYPEDEF,
	VO_end,
} verbose_options_t;

typedef int c99c_print_cb_t(c99tree_cc_t *ctx, void **nprint_ctx, const char *prefix, const c99c_node_t *node, print_state_t state);

struct c99tree_cc_s {
	/* parsing */
	c99c_node_t *AST;   /* Abstract Syntax Tree: the grammar */
	c99c_node_t *TOKT;  /* Token Tree: list of tokens */
	int close_yyin;
	char *c_typename;
	char *comment;
	int comment_alloced, comment_used;
	int node_uid_counter;
	c99c_scope_t *scope;
	void *scanner;
	int yydebug;
	FILE *yin; /* TODO: temp hack */
	char *error_hint;   /* textual hint from the parser - a pointer to the token for example */

	/* output options */
	int format_long;               /* true if longer, more verbose output should be generated */
	int verbose_options[VO_end];   /* where to output more detauls (see also: verbose_options_t) */
	char *loc_format;              /* format string for printing location */

	/* location related */
	c99c_loc_t *loc_current;            /* current location we are reading tokens at (top of the location stack) */
	c99c_loc_t loc_invalid;
	htsp_t *loc_files;
	htss_t *cpp_files;
	c99c_loc_t **loc_stack;
	int loc_alloced, loc_used;
	c99c_node_t *macro_expand_tt;      /* last token macro expanded on first level */

	/* output */
/* callback to print using a specific output format; !opening
   terminates the current node (after callback had been called for
   all children nodes recursively); return non-zero if no more processing is
   required for the subtree */
	c99c_print_cb_t *Nprint_cb;

	/* gentrex support */
	c99c_gtx_list_t *gtx_head, *gtx_tail;
	gentrex_map_az_t map, map_init;
	gentrex_api_t api;
};

/* === lib API === */

/* init a cc context - set default values, allocate some fileds */
void c99tree_cc_init(c99tree_cc_t *ctx);

/* uninit a cc context - free field allocations */
void c99tree_cc_uninit(c99tree_cc_t *ctx);


/* generic callback functions implemented by the caller */
void c99c_error(c99tree_cc_t *ctx, const char *s);  /* fatal error */
void c99c_werror(c99tree_cc_t *ctx, const char *s); /* this is an error we could actually survive */


/* print detailed error message to f (but do not exit) */
void c99c_error_fprintf(c99tree_cc_t *ctx, const char *s, FILE *f);

/* set the output callback in ctx from textual name in fmt
   return the callback or NULL on error (in which case ctx is not changed) */
c99c_print_cb_t *c99c_set_output_format(c99tree_cc_t *ctx, const char *fmt);

/* set verbosity flag in ctx by name
   return 0 on success, -1 on error */
int c99c_set_verbose(c99tree_cc_t *ctx, const char *name);


/* run the parser on yyin */
int c99c_parse(c99tree_cc_t *ctx);


extern const char *c99c_verbose_option_names[];
char *c99c_strclone(const char *in); /* strdup */


/* random prototypes for internal use */
int c99c_lex();
void c99c_set_in();
int c99c_lex_init();
int c99c_lex_destroy();


#endif
