#!/bin/bash

echo "Content-type: text/html"
echo ""

radio()
{
	local chk
	if test "$3" = "$2"
	then
		chk=" checked=\"true\""
	fi
	echo "<input type=\"radio\" name=\"$1\" value=\"$2\"$chk>"
}

fix_ltgt()
{
	sed "s/</\&lt;/g;s/>/\&gt;/g"
}

cmd_tree()
{
	echo "<H2>c99tree: The tree</H2>"
	if test -z "$QS_format"
	then
		QS_format=ind
	fi
	v=""
	if test ! -z "$QS_v"
	then
		v="$v -$QS_v"
	fi

	treecc -T$QS_format $v $c_tmp 2>&1  > $o_tmp

	case "$QS_format" in
		dot)
			echo "<svg width=10000 height=10000>"
			dot -Tsvg $o_tmp
			echo "</svg>"
			;;
		c)
			echo "<PRE>"
			fix_ltgt < $o_tmp | grep -v "^$"
			echo "</PRE>"
			;;
		*)
			echo "<PRE>"
			fix_ltgt < $o_tmp
			echo "</PRE>"
			;;
	esac


	case "$QS_format" in
	ind) 
		echo '
		<p>
		Format help: [] contains [node_uid:twin_uid]; node_uid is the unique
		ID of the current node, twin_uid is the uid of the same node in the twin
		subtree. () contains location information.
		';;
	esac
}

cmd_call()
{
	echo "<H2>statistics on function calls: $QS_format </H2>"
	case "$QS_format" in
		map|list)
			echo "<pre>"
			c99tree "call-$QS_format" "$c_tmp" ;
			echo "</pre>"
			;;
		graph)
			echo "<svg width=2000 height=600>"
			c99tree "call-graph" "$c_tmp" > $o_tmp
			dot -Tsvg $o_tmp
			echo "</svg>"
			;;
		*) echo "invalid call command."
	esac

}

cmd_globals()
{
	echo "<H2>statistics on $QS_format globals </H2>"
	echo "<pre>"
	case "$QS_format" in
		all|state|public) c99tree globals "--$QS_format" "$c_tmp" ;;
		*) echo "invalid globals command."
	esac
	echo "</pre>"
}

cmd_edit()
{
	export QS_prefix=`echo "$QS_prefix" | /home/igor2/C/libporty/trunk/src/porty/c99tree/url.sh`
	export QS_suffix=`echo "$QS_suffix" | /home/igor2/C/libporty/trunk/src/porty/c99tree/url.sh`


	echo "<H2> Edit $QS_format output </H2>"
	echo "<pre>"
	case "$QS_format" in
		wrapstmt) c99tree edit wrap_stmt --stmtprefix "$QS_prefix" --stmtsuffix "$QS_suffix" "$c_tmp" 2>&1 | fix_ltgt ;;
		*) echo "invalid edit command."
	esac
	echo "</pre>"
}

cmd_locdump()
{
	echo "<H2> Locdump </H2>"
	c99tree locdump --html "$c_tmp" 2>&1
#	 | fix_ltgt
}

qs=`echo "$QUERY_STRING" | tr "&" "\n"`


for n in $qs
do
    exp="QS_$n"
    export $exp
done

export QS_src=`echo "$QS_src" | /home/igor2/C/libporty/trunk/src/porty/c99tree/url.sh`

if test -z "$QS_src"
then
	export QS_src='
#include <stdio.h>
int gv1, gv2;
int main(int argc, char *argv[])
{
	static char *lv1;
	printf("Hello world!\n");
	return main(0, NULL);
}
'
fi

if test -z "$QS_cmd"
then
	QS_cmd="treecc_ind"
fi

echo "<html>"
echo "<body>"
echo "<H1> c99tree test page </H1>"
echo '
On this page you can try out c99tree on your C source
file without compilinf or installing anything on your system. Just copy&paste
your source into the textbox below, select a format and submit. You can #include
libc (an oldish version of gnu libc will be used)'
echo "<P>"
echo "<b>source:</b>"
echo "<form action=\"/cgi-bin/c99tree.cgi\" method=get>"
echo "<textarea id=\"src\" name=\"src\" cols=80 rows=20>$QS_src</textarea>"
echo "<P>"
echo "<table border=1 cellspacing=0 cellpadding=5>"
echo "	<tr>"
echo "	<th bgcolor=green>Command"
echo "	<th bgcolor=green>Options"

###
echo "	<tr>"
echo "	<td> <b>treecc</b> using the following output format:"
echo "	<table border = 0>"
echo "		<tr><td>&nbsp;<td> `radio cmd treecc_ind $QS_cmd` <td>ind"
echo "		<tr><td>&nbsp;<td> `radio cmd treecc_xml $QS_cmd` <td>xml"
echo "		<tr><td>&nbsp;<td> `radio cmd treecc_plist $QS_cmd` <td>plist"
echo "		<tr><td>&nbsp;<td> `radio cmd treecc_web3 $QS_cmd` <td>web3"
echo "		<tr><td>&nbsp;<td> `radio cmd treecc_json $QS_cmd` <td>json"
echo "		<tr><td>&nbsp;<td> `radio cmd treecc_perl $QS_cmd` <td>perl"
echo "		<tr><td>&nbsp;<td> `radio cmd treecc_awk $QS_cmd` <td>awk"
echo "		<tr><td>&nbsp;<td> `radio cmd treecc_c $QS_cmd` <td>abstract c source"
#echo "		<tr><td>&nbsp;<td> `radio cmd treecc_dot $QS_cmd` <td>graphviz"
echo "	</table>"
echo "	<td><table border = 0>"
echo "		<tr><td> <input type=\"checkbox\" name=\"v\" value=\"vtypedef\"> <td>expand typedef under CUSTOM_TYPE"
echo "	</table>"


###
echo "	<tr>"
echo "	<td> list <b>globals</b>:"
echo "	<table border = 0>"
echo "		<tr><td>&nbsp;<td> `radio cmd globals_all $QS_cmd` all: functions, variables, global or local static (the raw list)"
echo "		<tr><td>&nbsp;<td> `radio cmd globals_state $QS_cmd` global states: global or local static variables"
echo "		<tr><td>&nbsp;<td> `radio cmd globals_public $QS_cmd` global public variables and functions"
echo "	</table>"
echo "	<td><table border = 0>"
echo "		<tr><td> &nbsp;"
echo "	</table>"


###
echo "	<tr>"
echo "	<td> <b>function</b> calls:"
echo "	<table border = 0>"
echo "		<tr><td>&nbsp;<td> `radio cmd call_map $QS_cmd` <td>all: raw dump of function call information"
echo "		<tr><td>&nbsp;<td> `radio cmd call_list $QS_cmd` <td>list of internal calls"
echo "		<tr><td>&nbsp;<td> `radio cmd call_graph $QS_cmd` <td>graph of internal calls"
echo "	</table>"
echo "	<td><table border = 0>"
echo "		<tr><td>&nbsp;"
echo "	</table>"


###
echo "	<tr>"
echo "	<td> Complex <b>edit</b>:"
echo "	<table border = 0>"
echo "		<tr><td>&nbsp;<td> `radio cmd edit_wrapstmt $QS_cmd` <td>wrap statements between prefix and suffix"
echo "	</table>"
echo "	<td><table border = 0>"
echo "		<tr><td> prefix text: <td> <input name=\"prefix\" value=\"[!\">"
echo "		<tr><td> suffix text: <td> <input name=\"suffix\" value=\"!]\">"
echo "	</table>"

###
echo "	<tr>"
echo "	<td> c99tree debugging"
echo "	<table border = 0>"
echo "		<tr><td>&nbsp;<td> `radio cmd locdump $QS_cmd` locdump: 2-column location highlight"
echo "	</table>"
echo "	<td><table border = 0>"
echo "		<tr><td> &nbsp;"
echo "	</table>"


echo "</table>"
echo "<input type=\"submit\">"
echo "</form>"

QS_format=${QS_cmd##*_}
QS_cmd_=${QS_cmd%%_*}

if test ! -z "$QS_src"
then
	c_tmp=`tempfile -s .c`
	o_tmp=`tempfile`
	echo "$QS_src" > $c_tmp


	case "$QS_cmd_" in
		treecc) cmd_tree ;;
		call) cmd_call ;;
		globals) cmd_globals ;;
		edit) cmd_edit ;;
		locdump) cmd_locdump;;
		*) echo "Error: unknwon command"
	esac

	rm $c_tmp $o_tmp

	echo "<p><small>"
	c99tree-cc -V
	echo "</small>"
fi



echo "</body>"
echo "</html>"
